#!/bin/bash
#
# exiffixs3is
#
# Copyright (c)2007-2012, Akamai Haole, LLC. All rights reserved. You may freely
# use and distribute this script; however, this copyright notice must remain
# intact.
#
#
# Wrapper to ExifTool to "fix" Canon PowerShot S3 IS image files and place
# the correct ISO and Focal Length (35mm) values into the Exif data, 
# specifically for use in the Aperture software program. This script should
# be run from a Terminal window in Mac OS X.
#
# The ExifTool package (http://www.sno.phy.queensu.ca/~phil/exiftool/) must be
# properly installed and the location copied into in your PATH variable.
# This is easily accomplished through the installer provided with the
# ExifTool package.  ExifTool is owned and copyrighted by Phil Harvey.
#
# This author does not provide support for ExifTool, but I do encourage users
# of this script to support the ExifTool author/owner by donating to him for
# his excellent Exif utility.
#
# This script was written specifically for Mac OS X.  It was tested with 
# Mac OS X 10.4.10 and ExifTool 6.9.9 for Mac OS X.
#
# Revisions:
#   0.1.0 - Neil Sharp - Initial version (used ExifUtils)
#   0.2.0 - Neil Sharp - Changed Exif utility (uses ExifTool), cleanup, fixes
#
# Known Limitations:
#   1) ExifTool will round the focal length (up or down, depending on the value)
#      before placing it into the ExifIFD:FocalLengthIn35mmFormat tag within 
#      the picture file.
#   2) The ISO values can be very strange-looking if your camera is set to 
#      Auto ISO or Hi Auto ISO. The full-auto picture mode also uses one of 
#      these ISO settings. They are actual valid values, however.
#


################
# VARIABLES
################

DATE="2007/10/18"
VERSION="0.2.0"
DEBUG="OFF"
CURRENT_FILE=$1
EXIFTOOL=exiftool
AUTO_ISO_VALUE=""
BASE_ISO_VALUE=""
ACTUAL_ISO_VALUE=""
FOCAL_LENGTH_VALUE=""
FOCAL_LENGTH_35MM_VALUE=""
WHICH_EXIFTOOL=""
CHECK_ISO=""
CHECK_FOCAL_LENGTH_35MM=""
NEW_ISO=""
NEW_FOCAL_LENGTH_35MM=""

################
# FUNCTIONS
################

#
# Print copyright information
#
function printCopyright
{
  echo 
  echo "`basename $0` - v${VERSION} (${DATE})"
  echo
  echo "Copyright (c)2007, Sharp Focus Media (http://www.sharpfocusmedia.com)."
  echo "All rights reserved."
}

#
# Print usage of script
#
function printUsage
{
  echo "  Usage: `basename $0` <filename> [ q | quiet ]"
  echo "         where 'quiet' mode runs without the copyright information"
  echo
}

#
# Print an error message
#
function errorMsg
{
  echo
  echo "  ERROR: $1"
  echo
}


################
# MAIN
################

#
# Make sure we're not in quiet-mode
#
if [ "$2" != "q" -a "$2" != "quiet" ]; then
  printCopyright
fi

#
# Process error conditions
#
if [ "$CURRENT_FILE" = "h" -o "$CURRENT_FILE" = "?" ]; then
  printUsage 
  exit 0
fi

if [ "$CURRENT_FILE" = "" ]; then
  errorMsg "You must pass in the name of the file to process."
  printUsage
  exit -1
fi

if [ ! -f $CURRENT_FILE ]; then
  errorMsg "Specified file ($CURRENT_FILE) does not exist."
  exit -1
fi

#
# Process file
#

echo
echo "  Current File: $CURRENT_FILE"

WHICH_EXIFTOOL=`which $EXIFTOOL`
if [ -x "$WHICH_EXIFTOOL" ]; then
  AUTO_ISO_VALUE=`$EXIFTOOL -s -t -Canon:AutoISO $CURRENT_FILE | awk '{print $2}'`

  if [ "$DEBUG" == "ON" ]; then
    echo
    echo "    AUTO ISO = $AUTO_ISO_VALUE"
  fi

  BASE_ISO_VALUE=`$EXIFTOOL -s -t -Canon:BaseISO $CURRENT_FILE | awk '{print $2}'`

  if [ "$DEBUG" == "ON" ]; then
    echo "    BASE ISO = $BASE_ISO_VALUE"
  fi

  # Calculate the actual ISO value using bc
  ACTUAL_ISO_VALUE=`echo "scale=0; $BASE_ISO_VALUE * $AUTO_ISO_VALUE / 100" | bc`

  if [ "$DEBUG" == "ON" ]; then
    echo "  ACTUAL ISO = $ACTUAL_ISO_VALUE"
  fi

  # Get the real digital lens focal length
  FOCAL_LENGTH_VALUE=`$EXIFTOOL -s -t -FocalLength35efl $CURRENT_FILE | awk '{print $2}' | sed 's/mm//g'`

  if [ "$DEBUG" == "ON" ]; then
    echo
    echo "  FOCAL LENGTH (digital) = ${FOCAL_LENGTH_VALUE}mm"
  fi

  # ExifTool calculates the 35mm focal length
  FOCAL_LENGTH_35MM_VALUE=`$EXIFTOOL -s -t -FocalLength35efl $CURRENT_FILE | awk '{print $5}' | sed 's/mm)//g'`

  if [ "$DEBUG" == "ON" ]; then
    echo "     FOCAL LENGTH (35mm) = ${FOCAL_LENGTH_35MM_VALUE}mm"
  fi
else
  errorMsg "The $EXIFTOOL program is not in your path. Please reinstall ExifTool."
  exit -1
fi

  # Confirm that the files have not already been changed in a previous run
  CHECK_ISO=`$EXIFTOOL -s -t -ExifIFD:ISO $CURRENT_FILE | awk '{print $2}'`
  CHECK_FOCAL_LENGTH_35MM=`$EXIFTOOL -s -t -ExifIFD:FocalLengthIn35mmFormat $CURRENT_FILE | awk '{print $2}' | sed 's/mm//g'`

  if [ "$CHECK_ISO" != "" -o "$CHECK_FOCAL_LENGTH_35MM" != "" ]; then
    errorMsg "The file's values have already been altered. File was not changed.";
    exit -1;
  fi

if [ -x "$WHICH_EXIFTOOL" ]; then
  # Add ISO Exif information and add 35mm Focal Length Exif information
  if [ "$ACTUAL_ISO_VALUE" != "" -a "$FOCAL_LENGTH_35MM_VALUE" != "" ]; then
    $EXIFTOOL -q -ExifIFD:ISO=$ACTUAL_ISO_VALUE -ExifIFD:FocalLengthIn35mmFormat=$FOCAL_LENGTH_35MM_VALUE $CURRENT_FILE
  else
    errorMsg "Actual ISO value or focal length (35mm) value is invalid. File was not changed."
    exit -1
  fi

  NEW_ISO=`$EXIFTOOL -s -t -ExifIFD:ISO $CURRENT_FILE | awk '{print $2}'`
  NEW_FOCAL_LENGTH_35MM=`$EXIFTOOL -s -t -ExifIFD:FocalLengthIn35mmFormat $CURRENT_FILE | awk '{print $2}' | sed 's/mm//g'`

  echo
  echo "  New ISO value confirmed as: $NEW_ISO"
  echo "  New Focal Length (35mm equivalent) value confirmed as: ${NEW_FOCAL_LENGTH_35MM}mm"
  echo
else
  errorMsg "The $EXIFTOOL program is not in your path. Please reinstall ExifTool."
  exit -1
fi


################
# END OF SCRIPT
################
